#ifndef CONSTANT_H
#define CONSTANT_H


#include "petsc.h"


namespace Constant
{


	namespace Time
	{
		extern const PetscScalar& DeltaT; /**< \brief %Time step \f$ \Delta t \f$ */
		extern const PetscInt&    TBeginEq; /**< \brief Restart simulation time */
		extern const PetscInt&    TEndEq; /**< \brief Number of time steps during equilibrium */
		extern const PetscInt&    WriteEveryEq;/**< \brief Save every N time steps during equilibrium */
		extern const PetscInt&    PrintEveryEq;/**< \brief Print every N time steps during equilibrium */
		extern const PetscInt&    TBegin; /**< \brief Restart simulation time */
		extern const PetscInt&    TEnd; /**< \brief Number of time steps */
		extern const PetscInt&    WriteEvery; /**< \brief Save every N time steps */
		extern const PetscInt&    PrintEvery; /**< \brief Print every N time steps */
	}


	namespace Grid
	{
		extern const PetscInt&    Nx; /**< \brief Node number in x-direction \f$ N_x \f$ */
		extern const PetscInt&    Ny; /**< \brief Node number in y-direction \f$ N_y \f$ */
		extern const PetscInt&    Nz; /**< \brief Node number in z-direction \f$ N_y \f$ */
		extern const PetscScalar& Dx; /**< \brief %Grid spacing \f$ \Delta x \f$ */
		extern const PetscScalar* L; /**< \brief Domain length \f$ L = \left[ L_x \; L_y \; L_z \right]^\top \f$ */
		extern const PetscScalar* HL; /**< \brief Half domain length \f$ HL = \frac12 \left[ L_x \; L_y \; L_z \right]^\top  \f$ */
	}


	namespace Fluid
	{
		extern const PetscScalar& Xi; /**< \brief Length of the A/B diffuse interface \f$ \xi \f$ */
		extern const PetscScalar& Re; /**< \brief Reynolds number \f$ Re \f$ */
		extern const PetscScalar& We; /**< \brief Weber number \f$ We \f$ */
		extern const PetscScalar& Pe; /**< \brief Peclet number \f$ Pe \f$ */
		extern const PetscScalar& RhoRatio; /**< \brief %Fluid B to fluid A density ratio   \f$ \rho_B / \rho_A \f$ */
		extern const PetscScalar& EtaRatio; /**< \brief %Fluid B to fluid A viscosity ratio \f$ \eta_B / \eta_A \f$ */
	}


	namespace Foam
	{
// 		extern const PetscInt&    Nb; /** \brief Bubble number */
// 		extern const PetscInt&    ReadVZero; /**< \brief Read bubble volume flag */
// 		extern const PetscScalar& VZero; /**< \brief Initial bubble volume */
		extern const PetscScalar& BubbleGrowthCoef; /**< \brief Growth coefficient conserving bubble volume */
		extern const PetscScalar& BubbleRepulsionCoef; /**< \brief Repulsive coefficient */
		extern const PetscScalar* UFoam; /**< \brief %Foam velocity */
		extern const PetscInt&    ImportWall; /**< \brief Import wall flag */
	}


	namespace External
	{
		extern const PetscScalar* Gravity; /** \brief Gravity vector */
	}


	namespace Numerical
	{
		extern const PetscScalar& ThetaPhi; /**<\brief Theta rate, 0 = Explicit, 0.5 = Crank Nicholson, 1 = Implicit */
	}


	 /** \brief Read constants from input file */
	PetscErrorCode ReadInputFile();


}


#endif
