#include "petsc.h"
#include "PetscWrapper.h"

static char help[] = "Initialize bubble volume.\n\n";


PetscInt main( int argc, char* argv[] )
{
	PetscInitialize(&argc,&argv,(char*)0,help);
	PetscInt n = 10; PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL);
	PetscBool rnd = PETSC_FALSE; PetscOptionsGetBool(NULL,NULL,"-rnd",&rnd,NULL);
	PetscScalar V0 = 100; PetscOptionsGetScalar(NULL,NULL,"-V",&V0,NULL);
	PetscScalar *volumes; PetscMalloc1(n,&volumes);
	PetscRandom rand;
	PetscRandomCreate(PETSC_COMM_WORLD,&rand);
	PetscRandomSeed(rand);
	PetscRandomSetInterval(rand,0.5,1.5);
	for (PetscInt b=0; b<n; ++b)
	{
		PetscReal randomNumber = 1.0;
		if (rnd)
		{
			PetscRandomGetValueReal(rand,&randomNumber);
		}
		volumes[b] = V0 * randomNumber;
		PetscPrintf(PETSC_COMM_WORLD,"  V[%i] = %4.2f \n",b,volumes[b]);
	}
	MPI_Bcast(volumes,n,MPIU_SCALAR,0,PETSC_COMM_WORLD);
	PetscWrapper::WriteScalarInBinaryFormat(volumes,"initialBubbleVolume",n);
	PetscWrapper::WriteScalarInBinaryFormat(n,"bubbleNumber");
	PetscRandomDestroy(&rand);
	PetscFree(volumes);
	PetscFinalize();
	return(0);
}
