#include "Field2D.h"
#include "PetscWrapper.h"


PetscErrorCode Field2D::CreateField
(
	DM da,
	ScalarField &f,
	const std::string &name
)
{
	if (da)
	{
		PetscObjectReference((PetscObject)da);
	}
	f.da = da;
	PetscWrapper::CreateGlobalPetscVector(da,f.g,name);
	PetscWrapper::CreateLocalPetscVector(da,f.l);
	return(0);
}


PetscErrorCode Field2D::CreateField
(
	DM da,
	VectorField &F,
	const std::string &name,
	PetscInt n
)
{
	if (da)
	{
		PetscObjectReference((PetscObject)da);
	}
	F.da = da;
	PetscWrapper::CreateGlobalPetscVector(F.da,F.g,name,n);
	PetscWrapper::CreateLocalPetscVector (F.da,F.l,n);
	PetscMalloc1(n,&F.a);
	F.n = n;
	return(0);
}


PetscErrorCode Field2D::Create2DField
(
	DM da,
	VectorField &F,
	const std::string &name
)
{
	Field2D::CreateField(da,F,name,2);
	return(0);
}


PetscErrorCode Field2D::DestroyField
(
	ScalarField &f
)
{
	DMDestroy(&f.da);
	PetscWrapper::DestroyPetscVector(f.g);
	PetscWrapper::DestroyPetscVector(f.l);
	return(0);
}


PetscErrorCode Field2D::DestroyField
(
	VectorField &F
)
{
	DMDestroy(&F.da);
	PetscWrapper::DestroyPetscVector(F.g,F.n);
	PetscWrapper::DestroyPetscVector(F.l,F.n);
	PetscFree(F.a);
	return(0);
}


PetscErrorCode Field2D::GetFieldArray
(
	ScalarField &f
)
{
	PetscWrapper::GetPetscArray(f.da,f.g,f.a);
	return(0);
}


PetscErrorCode Field2D::GetFieldArray
(
	VectorField &F
)
{
	PetscWrapper::GetPetscArray(F.da,F.g,F.a,F.n);
	return(0);
}


PetscErrorCode Field2D::GetFieldArray
(
	VectorField &F,
	PetscInt i
)
{
	PetscWrapper::GetPetscArray(F.da,F.g[i],F.a[i]);
	return(0);
}


PetscErrorCode Field2D::RestoreFieldArray
(
	ScalarField &f
)
{
	PetscWrapper::RestorePetscArray(f.da,f.g,f.a);
	return(0);
}


PetscErrorCode Field2D::RestoreFieldArray
(
	VectorField &F
)
{
	PetscWrapper::RestorePetscArray(F.da,F.g,F.a,F.n);
	return(0);
}


PetscErrorCode Field2D::RestoreFieldArray
(
	VectorField &F,
	PetscInt i
)
{
	PetscWrapper::RestorePetscArray(F.da,F.g[i],F.a[i]);
	return(0);
}


PetscErrorCode Field2D::GetGhostedFieldArray
(
	ScalarField &f
)
{
	PetscWrapper::GetGhostedPetscArray(f.da,f.g,f.l,f.a);
	return(0);
}


PetscErrorCode Field2D::GetGhostedFieldArray
(
	VectorField &F
)
{
	PetscWrapper::GetGhostedPetscArray(F.da,F.g,F.l,F.a,F.n);
	return(0);
}


PetscErrorCode Field2D::GetGhostedFieldArray
(
	VectorField &F,
	PetscInt i
)
{
	PetscWrapper::GetGhostedPetscArray(F.da,F.g[i],F.l[i],F.a[i]);
	return(0);
}


PetscErrorCode Field2D::RestoreGhostedFieldArray
(
	ScalarField &f
)
{
	PetscWrapper::RestoreGhostedPetscArray(f.da,f.l,f.a);
	return(0);
}


PetscErrorCode Field2D::RestoreGhostedFieldArray
(
	VectorField &F
)
{
	PetscWrapper::RestoreGhostedPetscArray(F.da,F.l,F.a,F.n);
	return(0);
}


PetscErrorCode Field2D::RestoreGhostedFieldArray
(
	VectorField &F,
	PetscInt i
)
{
	PetscWrapper::RestoreGhostedPetscArray(F.da,F.l[i],F.a[i]);
	return(0);
}


PetscErrorCode Field2D::WriteFieldInBinaryFormat
(
	ScalarField &f,
	PetscInt t
)
{
	PetscWrapper::WritePetscVectorInBinaryFormat(f.g,t);
	return(0);
}


PetscErrorCode Field2D::WriteFieldInBinaryFormat
(
	VectorField &F,
	PetscInt t
)
{
	PetscWrapper::WritePetscVectorInBinaryFormat(F.g,t,F.n);
	return(0);
}


PetscErrorCode Field2D::WriteFieldInVTSFormat
(
	ScalarField &f,
	PetscInt t
)
{
	PetscWrapper::WritePetscVectorInVTSFormat(f.g,t);
	return(0);
}


PetscErrorCode Field2D::WriteFieldInVTSFormat
(
	VectorField &F,
	PetscInt t
)
{
	PetscWrapper::WritePetscVectorInVTSFormat(F.g,t,F.n);
	return(0);
}


PetscErrorCode Field2D::ReadFieldInBinaryFormat
(
	ScalarField &f,
	PetscInt t
)
{
	PetscWrapper::ReadPetscVectorInBinaryFormat(f.g,t);
	return(0);
}


PetscErrorCode Field2D::ReadFieldInBinaryFormat
(
	VectorField &F,
	PetscInt t
)
{
	PetscWrapper::ReadPetscVectorInBinaryFormat(F.g,t,F.n);
	return(0);
}


PetscErrorCode Field2D::SetField
(
	ScalarField &f,
	PetscScalar s
)
{
	PetscWrapper::SetPetscVector(f.g,s);
	return(0);
}


PetscErrorCode Field2D::SetField
(
	VectorField &F,
	PetscScalar s
)
{
	PetscWrapper::SetPetscVector(F.g,s,F.n);
	return(0);
}


PetscErrorCode Field2D::SetField
(
	VectorField &F,
	const PetscScalar *const S
)
{
	PetscWrapper::SetPetscVector(F.g,S,F.n);
	return(0);
}


PetscBool Field2D::AreFieldsEqual
(
	ScalarField &fa,
	ScalarField &fb,
	PetscScalar eps
)
{
	return PetscWrapper::ArePetscVectorsEqual(fa.g,fb.g,eps);
}


PetscErrorCode Field2D::CheckField
(
	ScalarField &f
)
{
	PetscWrapper::CheckPetscVector(f.g);
	return(0);
}


PetscErrorCode Field2D::CheckField
(
	VectorField &F
)
{
	PetscWrapper::CheckPetscVector(F.g,F.n);
	return(0);
}


PetscErrorCode Field2D::ComputeSumOfFieldElements
(
	ScalarField &f,
	PetscScalar &s
)
{
	PetscWrapper::ComputeSumOfPetscVectorElements(f.g,s);
	return(0);
}


PetscErrorCode Field2D::ComputeSumOfFieldElements
(
	VectorField &F,
	PetscScalar *S
)
{
	PetscWrapper::ComputeSumOfPetscVectorElements(F.g,S,F.n);
	return(0);
}


PetscErrorCode Field2D::ComputeArithmeticMeanOfFieldElements
(
	ScalarField &f,
	PetscScalar &m
)
{
	PetscWrapper::ComputeArithmeticMeanOfPetscVectorElements(f.g,m);
	return(0);
}


PetscErrorCode Field2D::ComputeArithmeticMeanOfFieldElements
(
	VectorField &F,
	PetscScalar *M
)
{
	PetscWrapper::ComputeArithmeticMeanOfPetscVectorElements(F.g,M,F.n);
	return(0);
}


PetscErrorCode Field2D::CopyFieldTo
(
	ScalarField &from,
	ScalarField &to
)
{
	PetscWrapper::CopyPetscVectorTo(from.g,to.g);
	return(0);
}


PetscErrorCode Field2D::CopyFieldTo
(
	VectorField &From,
	VectorField &To
)
{
// 	if (From.n != To.n)
// 	{
// 		PetscPrintf(PETSC_COMM_WORLD,"  Fields have unequal sizes (%i != %i)\n",From.n,To.n);
// 		PetscEnd();
// 	}
	PetscWrapper::CopyPetscVectorTo(From.g,To.g,To.n);
	return(0);
}


PetscErrorCode Field2D::SumVectorField
(
	VectorField &F,
	ScalarField &f
)
{
	PetscWrapper::SumArrayOfPetscVectors(F.g,f.g,F.n);
	return(0);
}
