#ifndef PHIEQUATION_H
#define PHIEQUATION_H


/** \file
 *  \brief CH solver */


#include "petsc.h"
#include "Context.h"
#include "SNESWrapper.h"


namespace PhiEquation
{
	/** \brief Computebubble volume
	 *  \param[out] ctx Solution context */
	PetscErrorCode ComputeBubbleVolume(Context::Ctx &ctx);


	/** \brief Compute square of the bubble volume
	 *  \param[out] ctx Solution context */
	PetscErrorCode ComputeSqrBubbleVolume(Context::Ctx &ctx);


	/** \brief Compute bulk potential
	 *  \param[out] ctx Solution context */
	PetscErrorCode ComputeBulk(Context::Ctx &ctx);


	/** \brief Compute repulsive potential
	 *  \param[out] ctx Solution context */
	PetscErrorCode ComputeRepulsion(Context::Ctx &ctx);


	/** \brief Compute right hand side of phi equations
	 *  \param[out] ctx Solution context */
	PetscErrorCode ComputeRHS(Context::Ctx &ctx);


	/** \brief Compute surface force
	 *  \note Kim, A continuous surface tension force formulation for diffuse-interface models, JCP, 2005
	 *  \param[out] ctx Solution context */
	PetscErrorCode ComputeSurfaceForce(Context::Ctx &ctx);


	/** \brief Addition all phi fields
	 *  \param[out] ctx Solution context */
	PetscErrorCode MakeFoam(Context::Ctx &ctx);


	/** \brief Compute chemical potentials
	 *  \param[out] ctx Solution context */
	PetscErrorCode ComputeChemicalPotential(Context::Ctx &ctx);


	/** \brief Form SNES function */
	PetscErrorCode FormSNESFunction(SNES,Vec,Vec,void*);

	/** \brief Form SNES left hand side matrix */
	PetscErrorCode FormSNESJacobian(SNES,Vec,Mat,Mat,void*);


}


#endif
