#ifndef UEQUATION_H
#define UEQUATION_H


/** \file
 *  \brief Flow velocity solver */


#include "petsc.h"
#include "Context.h"


namespace UEquation
{


	/** \brief Compute density and viscosity from smooth */
	PetscErrorCode ComputeEtaRho(Context::Ctx &ctx);


	/** \brief Compute convective acceleration */
	PetscErrorCode ComputeFConv(Context::Ctx &ctx,PetscInt j,PetscInt i);


	/** \brief Compute viscous acceleration */
	PetscErrorCode ComputeFViscAndFViscT(Context::Ctx &ctx,const PetscScalar *const invRho,PetscInt j,PetscInt i);


	/** \brief Compute capillary acceleration */
	PetscErrorCode ComputeFCap(Context::Ctx &ctx,const PetscScalar *const invRho,PetscInt j,PetscInt i);


	/** \brief Compute external acceleration */
	PetscErrorCode ComputeFExt(Context::Ctx &ctx,const PetscScalar *const invRho,PetscInt j,PetscInt i);


	/** \brief Compute right hand side of phi equations */
	PetscErrorCode ComputeRHS(Context::Ctx &ctx);


	PetscErrorCode ComputeDivRHS(Context::Ctx &ctx);


	PetscErrorCode SubstractGradP(Context::Ctx &ctx);


	PetscErrorCode SubstractMomentumExcess(Context::Ctx &ctx);


}


#endif
